package com.autentia.adictosaltrabajo.gae.persistence;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.google.appengine.api.datastore.DatastoreService;
import com.google.appengine.api.datastore.Entity;
import com.google.appengine.api.datastore.FetchOptions;
import com.google.appengine.api.datastore.Query;
import com.google.appengine.api.datastore.FetchOptions.Builder;
import com.google.appengine.api.datastore.Query.SortDirection;

public class TutorialUtils {

	private static final int FETCH_MAX_RESULTS = 10;
	
	/**
	 * Almacenamiento de un nuevo tutorial
	 * @param autor nombre del autor
	 * @param tituloTutorial titulo del tutorial
	 */
	public static void insert(final String autor, final String tituloTutorial) {
		// recuperacion del datastore
		final DatastoreService datastoreService = DSF.getDatastoreService();
		
		// creamos un nuevo tutorial y los insertamos en el datastore
		final Tutorial tutorial = new Tutorial(autor, tituloTutorial);		
		datastoreService.put(tutorial.getEntity());
	}
	
	/**
	 * Recuperación de los últimos 10 tutoriales (FETCH_MAX_RESULTS)
	 * @return una lista con los últimos 10 tutoriales
	 */
	public static List<Tutorial> getEntries() {
		// recuperación del datastore y configuracion de la consulta
		final DatastoreService datastoreService = DSF.getDatastoreService();
		final Query query = configureQuery();
		final FetchOptions fetchOptions = configureFetchOptions();

		// declaracion de un listado donde volcar los resultados
		final List<Tutorial> tutoriales = new ArrayList<Tutorial>();
		
		// lanzamiento de la consulta, la cual recupera entidades
		for (Entity entity: datastoreService.prepare(query).asList(fetchOptions)) {
			// conversion de las entidades a tutoriales
			tutoriales.add(convertEntityToTutorial (entity));	
		}		
		return tutoriales;
	}
	
	/**
	 * Conversion de entidad a tutorial
	 * @param entity entidad a ser convertida en tutorial
	 * @return tutorial procedente de una entidad
	 */
	private static Tutorial convertEntityToTutorial (final Entity entity) {
		final String autor = (String) entity.getProperty(Tutorial.AUTOR);
		final String tutorial = (String) entity.getProperty(Tutorial.TUTORIAL);
		final Date fecha = (Date) entity.getProperty(Tutorial.FECHA);
		return new Tutorial(autor, tutorial, fecha);
	}
	
	/**
	 * Configuración de la consulta a lanzar contra el datastore
	 * @return una consulta configurada
	 */
	private static Query configureQuery () {
		final Query query = new Query(Tutorial.TUTORIAL_ENTITY);
		query.addFilter(Tutorial.FECHA, Query.FilterOperator.NOT_EQUAL, null);
		query.addSort(Tutorial.FECHA, SortDirection.DESCENDING);		
		return query;
	}
	
	/**
	 * Configuración de las opciones de recuperacion de datos
	 * @return unas opciones de recuperacion de datos configuradas
	 */
	private static FetchOptions configureFetchOptions () {
		return Builder.withLimit(FETCH_MAX_RESULTS);	
	}
}
